#!/usr/bin/python
# -*- coding: utf-8 -*-
__author__ = "huangsheng"

import os, sys
reload(sys)
sys.setdefaultencoding('utf8')
from PyQt4.QtCore import QSettings, QObject ,QString
from PyQt4.QtCore import pyqtSignal
from ftplib import FTP, error_perm
from hashlib import md5
from utils.common_utils import CommonUtils
from utils.access_thread import AccessThread
from utils.enums import AUPSource, Signals, AptFunc, RUNLIST, LaunchMode
from model.upackage import UPackage
from utils.enums import Signals, AppType, AptFunc
from utils.log_manager import LogManager
import socket
import urllib
import urllib2
import getpass
import shutil
from xdg import BaseDirectory as xdg


import gettext
gettext.textdomain("kylin-update-manager")
_ = gettext.gettext

class KylinUpdateService(QObject):

    KUM_CONF_PATH = "/etc/kum.conf"
    UPDATE_INTRANET_URL = "archive.kylinos-intranet.cn"
    UPDATE_INTERNET_URL = "archive.kylinos.cn"
    UPDATE_FILE_PATH = "/kylin/KYLIN-ALL/upgrades"

    SSMAP_PATH = os.path.join(xdg.xdg_cache_home, "ss.map")
    SINFO_PATH= os.path.join(xdg.xdg_cache_home, "sinfo")
    BACKUP_URL = "192.168.137.137"

    available_url = ""
    available_source = ""
    task_mode = ""
    sinfo_file_name = ""
    is_sinfo_match = ""
    is_sinfo_down_success = ""
    os_version_source = ""
    os_version = ""
    os_codename = ""
    desktop_or_server = ""

    iname_list = []
    cname_list = []

    ftp = None
    mw = None
    at = None
    is_create=False

    oneAppmsg = pyqtSignal(dict)

    is_error = False

    def __init__(self, mw):
        QObject.__init__(self)

        # ftp等库的超时时间
        socket.setdefaulttimeout(8)

        self.available_source = AUPSource.NOSOURCE
        self.mw = mw
        self.at = AccessThread(self)
        self.is_create = True
        self.at.accessOver.connect(self.mw.slot_access_completed)
        self.at.oneAppMsg.connect(self.mw.slot_get_one_app)
        self.oneAppmsg.connect(self.mw.slot_get_one_app)
        self.connect(self.at, Signals.get_kylin_update_over, self.slot_check_os_update_over)
        self.connect(self, Signals.check_os_update_over, self.mw.switch_task)
        self.at.start()

        #self.desktop_or_server = mw.cu.desktop_or_server()
        #self.os_version = mw.cu.get_os_version()
        #self.os_codename = mw.cu.get_os_codename()
        #self.init_conf()

    def __del__(self):
        if(self.ftp != None):
            self.ftp.quit()

    #
    # 函数：读取/etc/kum.conf
    #
    def init_conf(self):
        if (os.path.isfile(self.KUM_CONF_PATH)):
            settings = QSettings(self.KUM_CONF_PATH, QSettings.IniFormat)
            settings.beginGroup("server_url")

            self.UPDATE_INTERNET_URL = str(settings.value("internet").toString()).strip()
            self.UPDATE_INTRANET_URL = str(settings.value("intranet").toString()).strip()
            self.BACKUP_URL = str(settings.value("backup_ip").toString()).strip()

    #
    # 函数：检测是否有可用的升级媒介
    #
    def check_available_medium(self):
        # 清除旧文件
        self.clean_tmp()

        # 从各个媒介中寻找可用的源，下载ss.map和sinfo
        if(self.check_medium_cdrom() == True):
            self.download_ssmap_from_cdrom()
            self.is_sinfo_match = self.get_matched_conf_file()
            self.is_sinfo_down_success = self.download_sinfo_from_cdrom()

        elif(self.check_medium_intranet_server() == True):
            self.download_ssmap_from_intranet()
            self.is_sinfo_match = self.get_matched_conf_file()
            self.is_sinfo_down_success = self.download_sinfo_from_intranet()

        elif(self.check_medium_internet_server() == True):
            self.download_ssmap_from_internet()
            self.is_sinfo_match = self.get_matched_conf_file()
            self.is_sinfo_down_success = self.download_sinfo_from_internet()

        else:
            return False

        return True

    #
    # 函数：删除可能残留的旧ss.map和sinfo
    #
    def clean_tmp(self):
        if(os.path.exists(self.SSMAP_PATH)):
            os.remove(self.SSMAP_PATH)
        if(os.path.exists(self.SINFO_PATH)):
            os.remove(self.SINFO_PATH)

    #
    # 函数：检测升级光盘是否可用
    #
    def check_medium_cdrom(self):
        mount_root = "/media/" + getpass.getuser()

        if os.path.isdir(mount_root):
            mount_dirs = os.listdir(mount_root)
            for one_dir in mount_dirs:
                one_dir_path = os.path.join(mount_root, one_dir)
                if os.path.isdir(one_dir_path):
                    files = None
                    try:
                        files = os.listdir(one_dir_path)
                    except Exception, e:
                        # 跳过权限异常的目录，继续扫描下一个
                        LogManager.ERR(e)
                        print e
                        continue
                    for one_file in files:
                        one_file_path = os.path.join(one_dir_path, one_file)
                        if os.path.isdir(one_file_path) == False:
                            if one_file == "ss.map":
                                self.available_url = "file://" + one_dir_path
                                self.available_source = AUPSource.DVDSOURCE
                                #print "找到光盘软件源."
                                LogManager.INFO(_("find CD Source."))
                                print(_("find CD Source."))
                                return True

        return False

    #
    # 函数：检测局域网升级服务器是否可用
    #
    def check_medium_intranet_server(self):
        if(self.check_one_server(self.UPDATE_INTRANET_URL) == True):
            #print "通过DNS找到局域网软件源."
            LogManager.INFO(_("find LAN Source by DNS."))
            print(_("find LAN Source by DNS."))
            return True
        else:
            if(self.check_one_server(self.BACKUP_URL) == True):
                #print("通过IP找到局域网软件源.")
                LogManager.INFO(_("find LAN Source by IP."))
                print(_("find LAN Source by IP."))
                return True
            else:
                return False

    #
    # 函数：检查服务器
    #
    def check_one_server(self, url):
        try:
            self.ftp = FTP()
            self.ftp.connect(url)
            self.ftp.login("", "")
            self.available_url = "ftp://" + url
            self.available_source = AUPSource.FTPSOURCE
            return True
        except:
            return False

    #
    # 函数：检测外网软件源是否可用
    #
    def check_medium_internet_server(self):
        url = "http://" + self.UPDATE_INTERNET_URL
        try:
            # password_mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
            # password_mgr.add_password(None, url, self.mw.service_key, self.mw.hash_pwd)
            # opener = urllib2.build_opener(urllib2.HTTPBasicAuthHandler(password_mgr))
            # urllib2.install_opener(opener)
            # kord源已开放，无需密码
            urllib2.urlopen(url + self.UPDATE_FILE_PATH + "/ss.map")
            self.available_url = url
            self.available_source = AUPSource.HTTPSOURCE
            #print "找到互联网软件源."
            LogManager.INFO(_("find Internet Source."))
            print(_("find Internet Source."))
            return True

        except Exception, e:
            LogManager.ERR(e)
            print e
            return False

    #
    # 函数：从光盘下载ss.map
    #
    def download_ssmap_from_cdrom(self):
        shutil.copyfile(self.available_url[7:] + "/ss.map", self.SSMAP_PATH)

    #
    # 函数：从局域网服务器下载ss.map
    #
    def download_ssmap_from_intranet(self):
        bufsize = 1024
        fp = open(self.SSMAP_PATH, "wb")
        self.ftp.retrbinary("RETR " + "pub/kylin/ss.map", fp.write, bufsize)
        fp.close()

    #
    # 函数：从互联网服务器下载ss.map
    #
    def download_ssmap_from_internet(self):
        # url = "http://" + self.mw.service_key + ":" + self.mw.hash_pwd + "@" + self.UPDATE_INTERNET_URL + "/ss.map"
        # kord源已开放，无需密码
        url = "http://" + self.UPDATE_INTERNET_URL + self.UPDATE_FILE_PATH + "/ss.map"
        urllib.urlretrieve(url, self.SSMAP_PATH)

    #
    # 函数：获得和本地系统匹配的软件源配置文件名
    #
    def get_matched_conf_file(self):
        settings = QSettings(self.SSMAP_PATH, QSettings.IniFormat)
        settings.beginGroup("key-source")
        keys = settings.childKeys()

        if(self.mw.service_key == "kylin"):
            # 社区版，直接使用默认sinfo
            self.sinfo_file_name = "sinfo.standard." + self.os_codename
            return True
        else:
            # 商业版，匹配sinfo
            for key in keys:
                if(str(key) == self.mw.service_key):
                    self.sinfo_file_name = "sinfo." + str(settings.value(key).toString()).strip()
                    return True
            else:
                # 没有匹配的文件，使用默认值
                self.sinfo_file_name = "sinfo.standard." + self.os_codename
                return False

    #
    # 函数：从光盘下载sinfo
    #
    def download_sinfo_from_cdrom(self):
        try:
            shutil.copyfile(self.available_url[7:] + "/" + self.sinfo_file_name, self.SINFO_PATH)
            return True
        except Exception, e:
            print(_("copy error : "), e)
            return False

    #
    # 函数：从局域网服务器下载sinfo
    #
    def download_sinfo_from_intranet(self):
        fp = open(self.SINFO_PATH, "wb")
        try:
            bufsize = 1024
            self.ftp.retrbinary("RETR " + "pub/kylin/" + self.sinfo_file_name, fp.write, bufsize)
            fp.close()
            return True
        except error_perm, e:
            print(_("download error : "), e)
            fp.close()
            return False

    #
    # 函数：从互联网服务器下载sinfo
    #
    def download_sinfo_from_internet(self):
        url = "http://" + self.UPDATE_INTERNET_URL + self.UPDATE_FILE_PATH
        try:
            # password_mgr = urllib2.HTTPPasswordMgrWithDefaultRealm()
            # password_mgr.add_password(None, url, self.mw.service_key, self.mw.hash_pwd)
            # opener = urllib2.build_opener(urllib2.HTTPBasicAuthHandler(password_mgr))
            # urllib2.install_opener(opener)
            ##kord源已开放，无需密码
            f = urllib2.urlopen(url + "/" + self.sinfo_file_name)
            with open(self.SINFO_PATH, "wb") as code:
                code.write(f.read())
            return True

        except Exception, e:
            LogManager.ERR(e)
            print e
            return False

    #
    # 函数：判断是否需要apt-get update
    #
    def check_need_apt_get_update(self):
        md5s_source = []

        start = False
        try:
            fp = open(self.SINFO_PATH, "rb")
            for line in fp:
                if(start == True):
                    if(line.find("[") != -1 or line.strip() == "" or line == None):
                        break
                    else:
                        md5s_source.append(line.strip())

                if(line.find("[inrelease.md5]") != -1):
                    start = True
            fp.close()
            md5s_local = self.get_local_inrelease_md5()

            for md5_source in md5s_source:
                for md5_local in md5s_local:
                    if (md5_source == md5_local):
                        break
                    else:
                        # 本地lists和软件源相比不全，需要apt-get update
                        print(_("InRelease not match : "), md5_source)
                        return True
            # 不需要apt-get update
            return False

        except:
            pass



    #
    # 函数：获取下载的sources.list
    #
    def get_sources(self):
        sourceslist = []

        start = False
        try:
            fp = open(self.SINFO_PATH, "rb")
            for line in fp:
                if(start == True):
                    if(line.find("[") != -1 or line.strip() == "" or line == None):
                        break
                    else:
                        # if self.available_url.startswith("http"):
                        #     url = self.available_url[7:]
                        #     sourceslist.append("deb http://" + self.mw.service_key + ":" + self.mw.hash_pwd + "@" + url + line.strip())
                        # else:
                        #kord源已开放，无需密码
                        sourceslist.append("deb " + self.available_url + line.strip())

                if(line.find("[sources.list]") != -1):
                    start = True
            fp.close()
            return sourceslist
        except:
            pass


    #
    # 函数：获取下载的important和crucial列表
    #
    def get_lists(self):
        iname_list = []
        cname_list = []
        start = False
        if os.path.isfile(self.SINFO_PATH):
            fp = open(self.SINFO_PATH, "rb")
            for line in fp:
                if(start == True):
                    if(line.find("[") != -1 or line.strip() == "" or line == None):
                        break
                    else:
                        if("," in line):  #字段含逗号，说明逗号后面是运行命令
                            line =  line.split(",")
                            if(line[1].strip() == "run"):
                                RUNLIST.append(line[0].strip()) # 逗号后面是run表示用软件名运行
                            else:
                                RUNLIST.append(line[1].strip()) # 否则逗号后面是运行命令
                            iname_list.append(line[0].strip())
                        else:
                            iname_list.append(line.strip())

                if(line.find("[important." + self.desktop_or_server + "]") != -1):
                    start = True
            fp.close()
            cname_list = []
            start = False
            try:
                fp = open(self.SINFO_PATH, "rb")
                for line in fp:
                    if (start == True):
                        if (line.find("[") != -1 or line.strip() == "" or line == None):
                            break
                        else:
                            if ("," in line):  # 字段含逗号，说明逗号后面是运行命令
                                line = line.split(",")
                                if (line[1].strip() == "run"):
                                    RUNLIST.append(line[0].strip())  # 逗号后面是run表示用软件名运行
                                else:
                                    RUNLIST.append(line[1].strip())  # 否则逗号后面是运行命令
                                cname_list.append(line[0].strip())
                            else:
                                cname_list.append(line.strip())

                    if (line.find("[crucial." + self.desktop_or_server + "]") != -1):
                        start = True
                fp.close()
                return (iname_list, cname_list)
            except:
                pass
        else:
            pass



    #
    # 函数：获取本地InRelease文件md5值
    #
    def get_local_inrelease_md5(self):
        lists_path = "/var/lib/apt/lists"
        files = os.listdir(lists_path)
        md5s_local = []

        for f in files:
            full_path = lists_path + "/" + f
            if(os.path.isfile(full_path) == True):
                if(full_path.endswith("InRelease")):
                    ff = open(full_path, "rb")
                    m = md5()
                    m.update(ff.read())
                    md5s_local.append(m.hexdigest())

        return md5s_local

    #
    # 函数：开始apt任务，检测是否有自我升级、系统升级
    #
    def check_update_type(self):
        try:
            (self.iname_list, self.cname_list) = self.get_lists()
            self.at.work_list.append(AptFunc.INIT)
            self.at.work_list.append(AptFunc.GET_KYLIN_UPDATE)
        except:
            if self.mw.launch_mode == LaunchMode.AUTOMATIC:
                self.mw.exit_app()
            self.mw.show_exit_widget(_("Missing sinfo file"))




    #
    # 函数：判断是否有可用的系统升级和更新器自我升级完成
    # ku_list:SP升级包列表
    # kum：自升级包
    #
    def slot_check_os_update_over(self, ku_list, kum):
        is_kum_update = False

        if(kum != None):
            is_kum_update = kum.is_upgradable

        if(len(ku_list) == 0):
            # 源中没有升级包
            self.emit(Signals.check_os_update_over, False, is_kum_update)
        else:
            for pkg in ku_list:
                name = pkg.name
                if(name.find(self.desktop_or_server) != -1):
                    version = pkg.candidate.version
                    osversion = version[:version.find('-',version.find('-') + 1)]
                    version_list = version.upper().split('-')
                    self.os_version_source = osversion.upper()
                    print self.os_version
                    print osversion.upper()
                    if osversion.upper().find("SP")!=-1:
                        os_version_source_numb = int(osversion[osversion.upper().find('SP') + 2:])

                        if(self.os_version.upper().find("SP") != -1):
                            os_version_numb = int(self.os_version[self.os_version.upper().find('SP') + 2:])
                        else:
                            os_version_numb = 0

                        if( self.os_version.upper().find('SP') == -1 and os_version_source_numb > 0):
                            # 找到可用的系统升级
                            upkg = UPackage(pkg)
                            new_version = osversion.upper()
                            self.at.fill_info_cn(upkg)
                            if is_kum_update == False:
                                if ("KYDROID" in version_list):
                                    self.oneAppmsg.emit(
                                        {"apptype": AppType.SP, "displayName": upkg.displayName, "name": upkg.name,
                                         "icon": "./res/SP_icon.png", "downloadSizeHuman": upkg.downloadSizeHuman,
                                         "downloadSize": upkg.downloadSize, "installedVersion": self.os_version,
                                         "candidateVersion": pkg.candidate.version, "description": upkg.description})
                                else:
                                    self.oneAppmsg.emit(
                                        {"apptype": AppType.SP, "displayName": upkg.displayName, "name": upkg.name,
                                         "icon": "./res/SP_icon.png", "downloadSizeHuman": upkg.downloadSizeHuman,
                                         "downloadSize": upkg.downloadSize, "installedVersion": self.os_version,
                                         "candidateVersion": new_version, "description": upkg.description})
                            self.emit(Signals.check_os_update_over, True, is_kum_update)
                            break
                        elif( self.os_version.upper().find('SP') != -1 and os_version_source_numb > os_version_numb ):

                            # 找到可用的系统升级
                            upkg = UPackage(pkg)
                            new_version = osversion.upper()

                            self.at.fill_info_cn(upkg)
                            if is_kum_update == False:
                                if ("KYDROID" in version_list):
                                    self.oneAppmsg.emit(
                                        {"apptype": AppType.SP, "displayName": upkg.displayName, "name": upkg.name,
                                         "icon": "./res/SP_icon.png", "downloadSizeHuman": upkg.downloadSizeHuman,
                                         "downloadSize": upkg.downloadSize, "installedVersion": self.os_version,
                                         "candidateVersion": upkg.candidateVersion, "description": upkg.description})
                                else:
                                    self.oneAppmsg.emit(
                                        {"apptype": AppType.SP, "displayName": upkg.displayName, "name": upkg.name,
                                         "icon": "./res/SP_icon.png", "downloadSizeHuman": upkg.downloadSizeHuman,
                                         "downloadSize": upkg.downloadSize, "installedVersion": self.os_version,
                                         "candidateVersion": new_version, "description": upkg.description})
                            self.emit(Signals.check_os_update_over, True, is_kum_update)
                            break
                    else:
                        self.emit(Signals.check_os_update_over, False, is_kum_update)
                        pass
                else:
                    # 源中没有系统升级包 / 本地和源不是同一个大版本 / 源中没有可更新的版本
                    self.emit(Signals.check_os_update_over, False, is_kum_update)

    #
    # 函数：获取系统本地版本和可升级的版本
    #
    def get_os_versions(self):
        return (self.os_version, self.os_version_source)

    #
    # 函数：开始获取和分类各种升级包
    #
    def start_access(self):
        self.at.work_list.append(AptFunc.GET_ALL_UPDATE)


def main():
    class mw:
        cu = CommonUtils()
        service_key = "750247"
        hash_pwd = "7f5831fbbb60350cb32e39d337fc7c22"

    kus = KylinUpdateService(mw())
    print kus.BACKUP_URL
    print kus.UPDATE_INTERNET_URL
    print kus.UPDATE_INTRANET_URL
    # kus.clean_tmp()
    # print kus.check_medium_cdrom()
    # print kus.available_url
    # kus.download_ssmap_from_cdrom()
    # kus.get_matched_conf_file()
    # print kus.sinfo_file_name
    # print kus.download_sinfo_from_cdrom()

    # print kus.check_available_medium()
    # print kus.sinfo_file_name
    # print "is match : ", kus.is_sinfo_match
    # print "is down success : ", kus.is_sinfo_down_success
    # print "is need apt update : ", kus.check_need_apt_get_update()
    # print kus.get_sources()


if __name__ == '__main__':
    main()
